/* $Id: attribute.cc,v 1.22 2001/06/06 13:36:20 martin Exp $ */
// Medoosa: Support for Documentation of C++ Programs
// Master's Thesis, Faculty of Mathematics and Physics, Charles University
// Martin Vidner, 2001
// Released under the terms of the GNU General Public License.
#include "attribute.h"
#include <stdlib.h>
#include <libxml/parser.h>
#include <libxml/tree.h>
#include "class.h"
#include "debug.h"


void MAssociation::InitUxf() {
  if (_uassoc == NULL) {
    _uassoc = xmlNewNode(NULL, IS_UTF8 "Association");

    _uend1 = xmlNewNode(NULL, IS_UTF8 "AssociationEnd");
    _uend2 = xmlNewNode(NULL, IS_UTF8 "AssociationEnd");
    // Do not add them yet, they have to be added in the propper 
    // order so that uxf-dia can connect them to the class boxes.
  }
}

int MAssociation::GetEndsUxf(xmlNodePtr *e1, xmlNodePtr *e2) {
  if (_getEndsCount % 2) {
    *e1 = _uend1;
    *e2 = _uend2;
  }
  else {
    *e1 = _uend2;
    *e2 = _uend1;
  }
  return ++_getEndsCount;
}

static void mXmlRemoveProp(xmlNodePtr n, xmlChar *p) {
  xmlAttrPtr prop = xmlSetProp(n, p, NULL);
  xmlRemoveProp(prop);
}

void MAssociation::SaveUxf(xmlNodePtr modelUxf) {
  _route.SaveDia(_uassoc);

  // swap if necessary
  xmlChar *qName1 = xmlGetProp(_uend1, IS_UTF8 "qname");
  xmlChar *qName2 = xmlGetProp(_uend2, IS_UTF8 "qname");
  if (xmlStrcmp(qName1, qName2) > 0) {
    swap(_uend1, _uend2);
  }
  mXmlRemoveProp(_uend1, IS_UTF8 "qname");
  mXmlRemoveProp(_uend2, IS_UTF8 "qname");
  free(qName2);
  free(qName1);

  xmlAddChild(_uassoc, _uend1);
  xmlAddChild(_uassoc, _uend2);

  xmlAddChild(modelUxf, _uassoc);
}

void MAssociation::LoadUxf(xmlNodePtr uassoc) {
  xmlNodePtr uchild;
  for (uchild = uassoc->children; uchild != NULL; uchild = uchild->next) {
    if (uchild->type == XML_ELEMENT_NODE) {
      if (!xmlStrcmp(uchild->name, IS_UTF8 "TaggedValue")) {
	xmlNodePtr node;
	string tag, value;
	node = GetTaggedNode(uchild, &tag);
	if (tag == "DiagramRoute") {
	  _route.MDiagramRoute::LoadDia(node);
	}
	else if (tag == "DiagramConnection1") {
	  GetTaggedValue(uchild, NULL, &value);
	  _route._connectionPoints[0] = atoi(value.c_str());
	}
	else if (tag == "DiagramConnection2") {
	  GetTaggedValue(uchild, NULL, &value);
	  _route._connectionPoints[1] = atoi(value.c_str());
	}
      }
    }
  }
}

void MAssociation::Diff(const MAssociation &old, const MAttribute &oldAttr, xmlNodePtr patchRoot) const {
  if (_route != old._route) 
    oldAttr.DiffRequestChange(patchRoot, "@dia:route", _route.SaveString());
}

// ---------------------------------------------------------------

MAttribute::MAttribute(const MAttribute &b) :
  MFeature(b),
  _initialValue(b._initialValue),
  _multiplicity(b._multiplicity),
  // _changeability?
  _assocAttr(b._assocAttr)
{
  _assocAttr._attribute = this;
}

MAttribute & MAttribute::operator = (const MAttribute &b) {
  if (&b != this) {
    MFeature::operator = (b);
    _initialValue = b._initialValue;
    _multiplicity = b._multiplicity;
    // _changeability?
    _assocAttr = b._assocAttr;
    _assocAttr._attribute = this;
  }
  return *this;
}

void MAttribute::LoadParseTree(xmlNodePtr cattr) {
  // name 
  xmlChar *cname;
  cname = xmlGetProp(cattr, IS_UTF8 "name");
  _name = CAST_BAD cname;
  free(cname);

  // type 
  list<string> tokens;
  tokens = LoadTokens(FindFirstTokenChild(cattr));

  list<string> dummyname, type, dummyvalue, dummyparams;
  dynamic_cast<MClass*>(_owner)->ParseDeclaration(tokens.begin(), tokens.end(),
			   dummyname, type, dummyvalue, dummyparams);
  ParseProperties(type);
  _type.Load(type);

  // initial value is swallowed by Ccdoc :(
  
  _visibility.LoadCtfx(cattr);

  _sourceInfo.LoadCtfx(cattr);
  LoadComments(cattr);
}

void MAttribute::SaveUxf(xmlNodePtr ownerUxf, xmlNodePtr modelUxf) const {
  xmlNodePtr uattr;

  if (_isAssocAttr) {
    MAssociation *assoc = _assocAttr.GetAssociation();
    xmlNodePtr uthisend, uotherend; // associationEnd
    int count; // is this the first or second end processed

    assoc->InitUxf();
    count = assoc->GetEndsUxf(&uthisend, &uotherend);

    // fill appropriate parts into both ends
    // careful what goes where 
    // DUH, the "proper" UXF order is mucked anyway

    // Type and Name
    // also set qname, it will be used by assoc->SaveUxf and then removed
    MModelElement *otherClass = _owner->GetElement(_assocAttr._otherClass);
    string otherClassQn = otherClass->GetQualifiedName();
    string thisClassQn = _owner->GetQualifiedName();
    string qualifiedRoleName = otherClassQn + "::" + _name;

    _sourceInfo.SaveUxfTo(uotherend);

    xmlNewChild(uotherend, NULL, IS_UTF8 "Type",BAD_CAST otherClassQn.c_str());
    xmlSetProp(uotherend, IS_UTF8 "qname", BAD_CAST qualifiedRoleName.c_str());
    if (_assocAttr._otherRole.empty()) {
      // one-way association, there is no Attribute at the other end
      xmlNewChild(uthisend, NULL, IS_UTF8 "Type",BAD_CAST thisClassQn.c_str());
      xmlSetProp(uthisend, IS_UTF8 "qname", BAD_CAST thisClassQn.c_str());
    }

    xmlNewChild(uotherend, NULL, IS_UTF8 "Name", BAD_CAST _name.c_str());

    // Qualifier
    if (!_assocAttr._qualifierTypes.empty()) {
      xmlNodePtr uqualif;
      uqualif = xmlNewChild(uotherend, NULL, IS_UTF8 "Qualifier", NULL);
      bool first = true;
      list<string>::const_iterator i;
      for(i =  _assocAttr._qualifierTypes.begin(); 
	  i != _assocAttr._qualifierTypes.end(); ++i) {
	if (!first) {
	  xmlAddChild(uqualif, xmlNewText(IS_UTF8 ","));
	}
	else {
	  first = false;
	}
	xmlAddChild(uqualif, xmlNewText(IS_UTF8 ":"));
	xmlAddChild(uqualif, xmlNewText(BAD_CAST i->c_str()));
      } 
    }

    // AggregationKind
    if (_assocAttr._aggregation != NONE) {
      const char *s = (_assocAttr._aggregation == AGGREGATE)? 
	"AggregationWhole" : "CompositionWhole";
      xmlNewChild(uthisend, NULL, IS_UTF8 "AggregationKind", IS_UTF8 s);
    }

    // Multiplicity
    xmlAddChild(uotherend, _multiplicity.SaveUxf());

    // isNavigable
    xmlNewChild(uotherend, NULL, IS_UTF8 "isNavigable", IS_UTF8 "true");

    // Visibility
    xmlAddChild(uotherend, _visibility.SaveUxf());

    // TargetScope
    // TODO: TargetScope is a UXF bug
    if (_ownerScope == CLASSIFIER)
      xmlNewChild(uotherend, NULL, IS_UTF8 "TargetScope",IS_UTF8 "classifier");

    // (Changeability)
    // Ordering
    if (_assocAttr._ordering != UNORDERED) {
      const char *s = (_assocAttr._ordering == ORDERED)? "ordered" : "sorted";
      xmlNewChild(uotherend, NULL, IS_UTF8 "Ordering", IS_UTF8 s);
    }

    // if we are the second or lone end, save the uassoc
    if (count == 2 || _assocAttr._otherRole.empty()) {
      assoc->SaveUxf(modelUxf);
    }
  } // _isAssocAttr
  else { // ordinary attribute 
    uattr = xmlNewNode(NULL, IS_UTF8 "Attribute");  

    _sourceInfo.SaveUxfTo(uattr);
    xmlNewChild(uattr, NULL, IS_UTF8 "Name", BAD_CAST _name.c_str());
    xmlAddChild(uattr, _type.SaveUxf(IS_UTF8 "Type"));
    xmlAddChild(uattr, _visibility.SaveUxf());
    if (!_initialValue.empty())
      xmlNewChild(uattr, NULL, IS_UTF8 "InitialValue", BAD_CAST _initialValue.c_str());
    // TODO: TargetScope is a UXF bug
    if (_ownerScope == CLASSIFIER)
      xmlNewChild(uattr, NULL, IS_UTF8 "TargetScope", IS_UTF8 "classifier");

    //_changeability

    xmlAddChild(uattr, _multiplicity.SaveUxf());

    xmlAddChild(ownerUxf, uattr);
  }
}

void MAttribute::LoadUxf(xmlNodePtr uDiamondAssocEnd) {
  _isAssocAttr = true;

  /* The AssociationEnd whose Type is the class to which this
     AssociationAttribute belongs is the diamond end */
  // get the other end
  xmlNodePtr n, uRoleAssocEnd;
  for (n = uDiamondAssocEnd->next; n != NULL; n = n->next) {
    if (n->type == XML_ELEMENT_NODE && 
	!xmlStrcmp(n->name, IS_UTF8 "AssociationEnd")) {
      break;
    }
  }
  if (n == NULL) {
    //try the other way
    for (n = uDiamondAssocEnd->prev; n != NULL; n = n->prev) {
      if (n->type == XML_ELEMENT_NODE && 
	  !xmlStrcmp(n->name, IS_UTF8 "AssociationEnd")) {
	break;
      }
    }
  }
  uRoleAssocEnd = n;
  assert(uRoleAssocEnd != NULL); // TODO: better error

  xmlNodePtr uchild;
  char *s;
  xmlChar *xs;

  for (uchild = uRoleAssocEnd->children; uchild!=NULL; uchild = uchild->next) {
    if (uchild->type == XML_ELEMENT_NODE) {
      if (!xmlStrcmp(uchild->name, IS_UTF8 "Name")) {
	s = CAST_BAD xmlNodeGetContent(uchild);
	_name = s;
	free(s);
	if (_name.empty()) {
	  // This is the dummy AA of a one-way association.
	  // The caller will delete us.
	  return;
	}
      }
      else if (!xmlStrcmp(uchild->name, IS_UTF8 "Type")) {
	s = CAST_BAD xmlNodeGetContent(uchild);
	_assocAttr._otherClass = s;
	free(s);	
      }
      else if (!xmlStrcmp(uchild->name, IS_UTF8 "Multiplicity")) {
	s = CAST_BAD xmlNodeGetContent(uchild);
	_multiplicity.LoadString(s);
	free(s);	
      }
      else if (!xmlStrcmp(uchild->name, IS_UTF8 "TaggedValue")) {
	string tag, value;
	GetTaggedValue(uchild, &tag, &value);
	if (tag == "SourceLine")
	  _sourceInfo._sourceLine = atoi(value.c_str());
      }
      else if (!xmlStrcmp(uchild->name, IS_UTF8 "Ordering")) {
	xs = xmlNodeGetContent(uchild);
	if (!xmlStrcmp(xs, IS_UTF8 "ordered"))
	  _assocAttr._ordering = ORDERED;
	else if (!xmlStrcmp(xs, IS_UTF8 "sorted"))
	  _assocAttr._ordering = SORTED;
	else
	  _assocAttr._ordering = UNORDERED;
	free(xs);
      }

      // Visibility and TargetScope not Diffable
      // TODO when it gets through dia-uxf: (Changeability)
    }
  }

  for (uchild = uDiamondAssocEnd->children; uchild != NULL; uchild = uchild->next) {
    if (uchild->type == XML_ELEMENT_NODE) {
      if (!xmlStrcmp(uchild->name, IS_UTF8 "AggregationKind")) {
	xs = xmlNodeGetContent(uchild);
	if (!xmlStrcmp(xs, IS_UTF8 "AggregationWhole"))
	  _assocAttr._aggregation = AGGREGATE;
	else if (!xmlStrcmp(xs, IS_UTF8 "CompositionWhole"))
	  _assocAttr._aggregation = COMPOSITE;
	else
	  _assocAttr._aggregation = NONE;
	free(xs);
      }
      else if (!xmlStrcmp(uchild->name, IS_UTF8 "Name")) {
	s = CAST_BAD xmlNodeGetContent(uchild);
	_assocAttr._otherRole = s;
	free(s);	
      }
    }
  }

  // Association
  MAssociation *assoc;
  xmlNodePtr uAssociation = uRoleAssocEnd->parent;
  if (_assocAttr._otherRole.empty()) {
    // a single AA, must take care of everthing:
    // create assn and hold it
    assoc = _assocAttr.InitAssociation();
    if (assoc == NULL) {
      cerr << "Error, cannot associate class '"
	   << _owner->GetQualifiedName() 
	   << "' with class '" 
	   << _assocAttr._otherClass <<"'.\n";
    }
    else {
      // load it
      assoc->LoadUxf(uAssociation);
    }
  }
  else {
    // only process the association after both AssocAttrs have been loaded
    bool flag = (xmlHasProp(uAssociation, IS_UTF8 "flag") != NULL);
    if (flag) {
      // ok, both are here.
      // create assn and assign it to the proper AA
      assoc = _assocAttr.InitAssociation();
      if (assoc == NULL) {
	cerr << "Error, cannot associate class '"
	     << _owner->GetQualifiedName() 
	     << "' with class '" 
	     << _assocAttr._otherClass <<"'.\n";
      }
      else {
	// load it
	assoc->LoadUxf(uAssociation);
      }
    }
    else {
      // leave the work for the other attribute
      xmlSetProp(uAssociation, IS_UTF8 "flag", IS_UTF8 "1");
    }
  }
}

void MAttribute::Diff(const MModelElement &old, xmlNodePtr patchRoot) const {
  // the old element has to be of the same type
  const MAttribute & oldAttr = dynamic_cast<const MAttribute &>(old);

  // multiplicity
  if (_multiplicity != oldAttr._multiplicity) {
    oldAttr.DiffRequestChange(patchRoot,"@uml:mul",_multiplicity.SaveString());
  }

  if (_isAssocAttr) {
    if (!oldAttr._isAssocAttr) {
      cerr << "Too radical a change of " << GetQualifiedName() << endl;
    }
    else {
      _assocAttr.Diff(oldAttr._assocAttr, patchRoot);
    }
  }
  //TODO _changeability
}

void MAttribute::ProcessDirectives() {
  Directives::iterator i;
  MAssociation *assoc = NULL;

  if (mDebug.IsEnabled('a')) 
    cerr << GetQualifiedName() << endl;

  // analyze the type
  MTypeAnalysis analysis = _type.Analyze();

  // what is the other class and role
  _assocAttr._otherClass = analysis._targetType;
  i = _directives.find("@uml:other");
  if (i != _directives.end()) {
    string dvalue = i->second;
    string otherClass;
    if (StringSplit(dvalue, ",", _assocAttr._otherRole, otherClass)) {
      // if a comma is there, assign
      _assocAttr._otherClass = otherClass;
    }
  }
  _isAssocAttr = !_assocAttr._otherClass.empty();

  // init the association at either end
  if (_isAssocAttr)
    assoc = _assocAttr.InitAssociation();

  // use the analysis results ...
  _multiplicity = analysis._multiplicity;
  //_changeability
  if (assoc != NULL) {
    _assocAttr._qualifierTypes = analysis._qualifierTypes;
    _assocAttr._aggregation = analysis._aggregation;
    _assocAttr._ordering = analysis._ordering;
  }

  // ... then override them by directives used
  for (i = _directives.begin(); i != _directives.end(); ++i) {
    string dname = i->first;
    string dvalue = i->second;

    if (dname == "@uml:mul") {
      _multiplicity.LoadString(dvalue);
    }
    else if (dname == "@uml:qualif") {
      // TODO: use it when UXF and Dia are ready
    }
    else if (dname == "@uml:order") {
      MOrderingKind ordering;
      if (dvalue == "ordered")
	ordering = ORDERED;
      else if (dvalue == "sorted")
	ordering = SORTED;
      else
	ordering = UNORDERED;

      _assocAttr._ordering = ordering;
    }
    else if (dname == "@uml:aggreg") {
      _assocAttr._aggregation = AGGREGATE;
      // TODO? connect to the other end, ensure that it has NONE
    }
    else if (dname == "@uml:compo") {
      _assocAttr._aggregation = COMPOSITE;
      // TODO? connect to the other end, ensure that it has NONE
    }
    else if (dname == "@dia:route") {
      if (assoc != NULL) {
	assoc->_route.LoadString(dvalue);
      }
    } // if @dia:route

  } // for all directives
}

// ---------------------------------------------------------------

MAssociationAttribute::MAssociationAttribute(MAttribute *a): 
  _attribute(a), 
  _aggregation(NONE),
  _ordering(UNORDERED),
  _association(NULL) 
{}

MAssociationAttribute::~MAssociationAttribute() {
  if (_association != NULL)
    delete _association;
}

MClass * MAssociationAttribute::GetOtherClass() const {
  MModelElement *cls_e = _attribute->_owner->GetElement(_otherClass);
  if (cls_e == NULL) {
    // not found: a "foreign" class, probably from a library
    
    // design decision: the only boxes in the diagram are this
    // project's classes
    return NULL;
  }
  else {
    return dynamic_cast<MClass *>(cls_e);
  }
}

MAttribute * MAssociationAttribute::GetHolder() const {
  MClass *cls = GetOtherClass();
  MAttribute * holder;
  if (cls == NULL) {
    // noone - no association
    holder =  NULL;
  }
  else {
    if (_otherRole.empty()) {
      // one-way association, we are the holder
      holder = _attribute;
    }
    else {
      // compare fully qualified attribute names, the first one wins
      string otherQn = cls->GetQualifiedName();
      string thisQn = _attribute->_owner->GetQualifiedName();
      int cmp = otherQn.compare(thisQn);
      if (mDebug.IsEnabled('a')) 
	cerr << otherQn << " <=> " << thisQn << " : " << cmp << endl;

      if (cmp == 0)
	cmp = _otherRole.compare(_attribute->_name);

      if (cmp < 0) 
	holder = dynamic_cast<MAttribute *>(cls->GetElement(_otherRole));
      else 
	holder = _attribute;      
    }
  }
  return holder;
}

MAssociation * MAssociationAttribute::InitAssociation() {
  // establish a relationship with the other class
  MAssociation *a;

  MAttribute * holder = GetHolder();
  if (holder == NULL) {
    a = NULL;
  }
  else {
    // if it is not initizalized already, create it
    a = holder->_assocAttr._association;
    if (a == NULL) {
      holder->_assocAttr._association = a = new MAssociation;
    }
  }
  return a;
}

MAssociation * MAssociationAttribute::GetAssociation() const {
  MAttribute *holder = GetHolder();
  MAssociation *a = holder->_assocAttr._association;
  return a;
}

void MAssociationAttribute::Diff(const MAssociationAttribute &old, 
				 xmlNodePtr patchRoot) const {

  // other role
  if (_otherRole != old._otherRole || _otherClass != old._otherClass) {
    string other = _otherRole + "," + _otherClass; 
    old._attribute->DiffRequestChange(patchRoot, "@uml:other", other);
  }

  // aggregation
  MAggregationKind ag = _aggregation;
  MAggregationKind oag = old._aggregation;
  if (ag != oag) {
    // delete the old value
    if (oag == AGGREGATE) 
      old._attribute->DiffRequestChange(patchRoot, "@uml:aggreg", "");
    else if (oag == COMPOSITE)
      old._attribute->DiffRequestChange(patchRoot, "@uml:compo", "");
    
    // insert the new one
    if (ag == AGGREGATE) 
      old._attribute->DiffRequestChange(patchRoot, "@uml:aggreg", " ");
    else if (ag == COMPOSITE)
      old._attribute->DiffRequestChange(patchRoot, "@uml:compo", " ");
  }
  
  // ordering
  if (_ordering != old._ordering) {
    char *s[] = { "unordered", "ordered", "sorted" };
    assert(_ordering <= SORTED);
    old._attribute->DiffRequestChange(patchRoot, "@uml:order", s[_ordering]);
  }
  
  // route
  MAssociation *as = _association;
  MAssociation *oas = old._association;
  if (as == NULL) {
    if (oas != NULL) {
      // association deleted (probably merged with another one-way assoc)
      old._attribute->DiffRequestChange(patchRoot, "@dia:route", "");
    }
  }
  else {
    if (oas == NULL) {
      cerr << "New association, ugh.";
    }
    else 
      as->Diff(*oas, *old._attribute, patchRoot);
  }
}

