// -*- C++ -*-
// Medoosa: Support for Documentation of C++ Programs
// Master's Thesis, Faculty of Mathematics and Physics, Charles University
// Martin Vidner, 2001
// Released under the terms of the GNU General Public License.
/* $Id: attribute.h,v 1.16 2001/04/23 16:10:28 martin Exp $ */
#ifndef __M_ATTRIBUTE_H
#define __M_ATTRIBUTE_H

#include <list>
#include <string>
#include "common.h"
#include "type.h"
#include "element.h"
#include "feature.h"
#include "diagram.h"
#include "umltypes.h"

class MClass;
class MAttribute;
// ---------------------------------------------------------------

/** @dia:pos -60,81 */
class MAssociation {
  friend class MAttribute;
public:
  MAssociation() : _uassoc(NULL), _getEndsCount(0) {}

  /** Saving is more complex than with other elements because we have
      to wait until both ends have provided their data. */
  void InitUxf();
  /** Provides the ends' nodes, 
   * alternating the order after each call (per instance).
   * @returns the number of calls (to this instance) including this one */ 
  int GetEndsUxf(xmlNodePtr *e1, xmlNodePtr *e2);
  void SaveUxf(xmlNodePtr modelUxf);

  void LoadUxf(xmlNodePtr unode);
  void Diff(const MAssociation &old, const MAttribute &oldAttr, xmlNodePtr patchRoot) const;

private:
  /** @dia:route 27,0;h,-36.4144,90.3,-33,87,-31 */
  /** @uml:mul 1 */
  MAssociationRoute _route;
  /** Have to keep it for the time period between both ends are saved */
  xmlNodePtr _uassoc;
  int _getEndsCount;
  xmlNodePtr _uend1;
  xmlNodePtr _uend2;
};

/** 
 * Similar to, but not the same as UML AssociationEnd.
 *
 * Two AssociationAttributes correspond to two AssociationEnds but
 * member variables of an AE are separated between the two AA's (and
 * vice versa). 
 * A one-way association has two AEs but only a single AA.
 */
/** @dia:pos -61,62 */
class MAssociationAttribute {
  //  friend class MAttribute;
public:
  MAssociationAttribute(MAttribute *a);
  ~MAssociationAttribute();
  /*
  MModelElement *Clone() const {return new MAssociationAttribute(*this);}
  */
  MAssociation * GetAssociation() const;
  /** If the _otherClass exists, return the Association (creating it,
      at the proper attribute, if it doesn't exist yet). Otherwise,
      return NULL. */
  MAssociation * InitAssociation();

  void Diff(const MAssociationAttribute &old, xmlNodePtr patchRoot) const;

  /** @dia:route 0,6;h,-61,62,-55,59,-48.9344,56.6 */
  /** @uml:other _assocAttr,MAttribute */
  MAttribute *_attribute;
  /** This is an oversimplification: in UML, Qualifier's type is a
      list of Attributes. But we have only their types and besides,
      UXF and Dia do not support them either so this is good
      enough. */
  list<string> _qualifierTypes; //TODO: (q)name
  MAggregationKind _aggregation;
  // bool _isNavigable; // is implicit;
  MOrderingKind _ordering; // unsupported by Dia

  string _otherClass;
  /** The name of the other AssociationAttribute that participates 
   * in this association.
   *
   * May be empty, meaning a one-way association. */
  string _otherRole;

private:
  /** @returns The MClass designated by _otherClass or NULL if not found */
  MClass * GetOtherClass() const;
  /** Decides whether the association is held by this AA or the one
   * designated by _otherRole or by noone.
   * 
   * If the other class is not found, noone wins (no association will
   * be created) and NULL is returned.  This attribute wins if _otherRole is
   * empty.  Otherwise, the attributes' qualified names are compared
   * and the first wins. */
  MAttribute * GetHolder() const;

  /** Only one end owns the association data. The other uses GetAssociation */
  /** @dia:route 2,7;h,-36.4144,81,-36,73.4,-40.808 */
  /** @uml:mul 0..1 */
  MAssociation * _association;
};

/** @dia:pos -60,46 */
/** @dia:route MFeature;v,-22.48,32.2,43,-48.9344,46 */
class MAttribute : public MFeature {
  friend class MAssociationAttribute;
public:
  MAttribute(MNamespace *ns): MFeature(ns), _assocAttr(this) {}
  MAttribute(const MAttribute &);
  MAttribute & operator = (const MAttribute &);
  MModelElement *Clone() const {return new MAttribute(*this);}
  void LoadParseTree(xmlNodePtr cnode);

  virtual void SaveUxf(xmlNodePtr ownerUxf, xmlNodePtr modelUxf) const;
  /** @param unode The diamond AssociationEnd. The role AE has to be
      found and used too. */
  virtual void LoadUxf(xmlNodePtr unode);
  virtual void Diff(const MModelElement &old, xmlNodePtr patchRoot) const;
  virtual void ProcessDirectives();
private:
  string _initialValue;
  /** @dia:route 0,2;h,-60,46,-63,52,-69.96 */
  /** @uml:mul 1 */
  MMultiplicity _multiplicity;
  //TODO:  _changeability?

  bool _isAssocAttr;  
  /** @uml:other _attribute,MAssociationAttribute */
  MAssociationAttribute _assocAttr; // abbreviated from _associationAttribute
};

#endif
