// -*- C++ -*-
/* $Id: class.h,v 1.23 2001/04/23 16:10:29 martin Exp $ */
// Medoosa: Support for Documentation of C++ Programs
// Master's Thesis, Faculty of Mathematics and Physics, Charles University
// Martin Vidner, 2001
// Released under the terms of the GNU General Public License.
#ifndef __M_CLASS_H
#define __M_CLASS_H

#include <list>
#include <string>
#include <libxml/tree.h>
#include "attribute.h"
#include "operation.h"
#include "parameter.h"
#include "diagram.h"
#include "element.h"
#include "namespace.h"
#include "traits.h"

class MClass;

/** Reference (by name, for now) to a superclass */
/** @dia:pos 15,87 */
class MGeneralization {
public:
  MGeneralization(MClass *child) : _child(child) {}

  void LoadTokens(list<string>::iterator b, list<string>::iterator e);
  void LoadUxf(xmlNodePtr ugeneralization);
  xmlNodePtr SaveUxf() const;
  
private:
  friend class MClass;

  string _parent; 
  /** @uml:mul 1 */
  /** @uml:other _superClasses,MClass */
  MClass *_child;

  bool _isVirtual;
  /** @dia:route 5,7;h,15,94.4,-67,47,-72.3232 */
  /** @uml:mul 1 */
  MVisibilityToken _visibilityToken;

  /** @dia:route 7,12;h,-6.96,79,-4,90.7,15 */
  /** @uml:mul 1 */
  MDiagramRoute _route;
  //prohibit
  MGeneralization();
  //MGeneralization(const MGeneralization&);
  //MGeneralization& operator = (const MGeneralization&);
};

// ---------------------------------------------------------------

enum MClassKind {
  ORDINARY,
  POINTER,
  CONTAINER
};

// 
/** @dia:pos -60,22 */
/** @dia:route MModelElement;v,-17.2072,17.8,20,-52.5704,22 */
class MClassStub : public MModelElement {
public:
  MClassStub(MNamespace *ns) : MModelElement("", ns) {} //OK?

  MClassKind _classKind;
  // In orthodox OO, this would be MClassTraits * -- an overkill.
  // We simply ignore unused parts of the class.
  /** @dia:route 0,2;h,-60,22,-90,17,-143.141 */
  MContainerTraits _containerTraits;
};

// ---------------------------------------------------------------

/** @dia:pos 3,55 */
/** @dia:route MNamespace;v,26.944,36,38,20.1256,55 */
class MClass : public MNamespace {
public:
  MClass(MNamespace *ns);
  MClass(const MClass&);

  static string LoadName(xmlNodePtr cnode);

  virtual MModelElement * Clone() const {return new MClass(*this);}
  virtual void LoadParseTree(xmlNodePtr cnode, const string &fileName);
  virtual void SaveUxf(xmlNodePtr ownerUxf, xmlNodePtr modelUxf) const;
  virtual void LoadUxf(xmlNodePtr unode);
  void LoadAssocAttrUxf(xmlNodePtr uDiamondAssocEnd);
  virtual void Diff(const MModelElement &old, xmlNodePtr patchRoot) const;
  virtual void ProcessDirectives();


  /** */
  void ParseDeclaration(list<string>::iterator b, list<string>::iterator e, 
			list<string> &name, list<string> &type, list<string> &value, list<string> &params);

  /** If diagram position is not specified, specify it.
      @param count Number of classes whose position had to be forced this way 
  */
  void EnsureDiagramPosition(int &count);

  /** Abstract operations call this to notify the class. */
  void SetAbstract() {_isAbstract = true;}
  virtual string SourceFile() const;
private:
  /** @dia:route 2,2;h,37.2512,55,56,87,48.7664 */
  /** @uml:other _child,MGeneralization */
  list<MGeneralization> _superClasses;
  /** @dia:route 3,11;h,3,55.7,2,65.7,-6.2336 */
  /** @uml:order ordered */
  list<MParameter> _templateParameters;
  /** Default is false, set only through SetAbstract() */
  bool _isAbstract;

  /** Position in the diagram. Dia specific. */
  /** @dia:route 7,2;h,37.2512,69.6,39,75,30.04 */
  /** @uml:mul 1 */
  MDiagramPoint _diagramPosition;

  // prohibit
  //for diff-uxf :(  MClass();
  MClass& operator = (const MClass&);
};

#endif
