/* $Id: common.cc,v 1.15 2002/03/17 19:09:36 martin Exp $ */
// Medoosa: Support for Documentation of C++ Programs
// Master's Thesis, Faculty of Mathematics and Physics, Charles University
// Martin Vidner, 2001
// Released under the terms of the GNU General Public License.
#define _ISOC99_SOURCE
// snprintf
#include <stdlib.h>
#include "common.h"
#include <ctype.h>
#include <libxml/parser.h>

void ForwardSkipBrackets(list<string>::iterator &i, char left, char right, list<string>::iterator end) {
  if (i != end && * i->c_str() == left) {
    int nesting = 0;
    char c;
    do {
      c = * i->c_str();
      if (c == left)
	++nesting;
      else if (c == right)
	--nesting;

      ++i;
    } while (nesting > 0 && i != end);
  }
}

void BackSkipBrackets(list<string>::iterator &i, char left, char right, list<string>::iterator begin) {
  if (PrevChar(i,begin) == right) {
    int nesting = 0;
    char c;
    do {
      c = PrevChar(i, begin);
      if (c == right)
	++nesting;
      else if (c == left)
	--nesting;

      --i;
    } while (nesting > 0);
  }
}

xmlNodePtr FindFirstTokenChild(xmlNodePtr node) {
  xmlNodePtr tokens;
  tokens = node->children;
  while (tokens != NULL && 
	 !(tokens->type == XML_ELEMENT_NODE && 
	   !xmlStrcmp(tokens->name, IS_UTF8 "tokens")))
    tokens = tokens->next;
  return tokens;
}

list<string> LoadTokens(xmlNodePtr tokens)
{
  list<string> sl;
  xmlNodePtr tokennode;
  xmlChar *token;

  for (tokennode=tokens->children; tokennode!=NULL; tokennode=tokennode->next) {
    if (tokennode->type == XML_ELEMENT_NODE) {
      token = xmlGetProp(tokennode, IS_UTF8 "value");
      sl.push_back(CAST_BAD token);
      free(token);
    }
  }

  return sl;
}

xmlNodePtr NewTaggedValue(xmlChar *tag, xmlChar *value) {
  xmlNodePtr unode;
  unode = xmlNewNode(NULL, IS_UTF8 "TaggedValue");
  xmlNewChild(unode, NULL, IS_UTF8 "Tag", tag);
  xmlNewChild(unode, NULL, IS_UTF8 "Value", value);
  return unode;
}

void GetTaggedValue(xmlNodePtr tvnode, string *tag, string *value) {
  xmlNodePtr child;
  char *s;
  for (child = tvnode->children; child != NULL; child = child->next) {
    if (child->type == XML_ELEMENT_NODE) {
      if (tag && !xmlStrcmp(child->name, IS_UTF8 "Tag")) {
	s = CAST_BAD xmlNodeGetContent(child);
	*tag = s;
	free(s);
      }
      else if (value && !xmlStrcmp(child->name, IS_UTF8 "Value")) {
	s = CAST_BAD xmlNodeGetContent(child);
	*value = s;
	free(s);
	// we have all we need
	break;
      }
    }
  }
}

xmlNodePtr GetTaggedNode(xmlNodePtr tvnode, string *tag) {
  xmlNodePtr child;
  xmlNodePtr gchild = NULL;
  char *s;
  for (child = tvnode->children; child != NULL; child = child->next) {
    if (child->type == XML_ELEMENT_NODE) {
      if (tag && !xmlStrcmp(child->name, IS_UTF8 "Tag")) {
	s = CAST_BAD xmlNodeGetContent(child);
	*tag = s;
	free(s);
      }
      else if (!xmlStrcmp(child->name, IS_UTF8 "Value")) {
	for (gchild = child->children; gchild != NULL; gchild = gchild->next) {
	  if (gchild->type == XML_ELEMENT_NODE) 
	    break;
	}
	// we have all we need
	break;
      }
    }
  }
  return gchild;
}

static inline bool IsWordChar(char c) {
  return isalpha(c) || c == '_';
}

string Detokenize(const list<string> & sl) {
  string str;
  bool wordSeen = false;
  list<string>::const_iterator i;
  string::const_iterator sb, se;

  for (i = sl.begin(); i != sl.end(); ++i) {
    sb = i->begin();
    se = i->end();
    assert (sb != se);

    if (wordSeen && IsWordChar(*sb))
      str += ' ';

    str += *i;

    --se; 
    // point to the last character
    wordSeen = IsWordChar(*se);
  }
  return str;
}

bool MVisibilityToken::Read(const char *s, bool tla) 
{
  bool success = true;
  if (tla) {
    if (!strcmp(s, "pub"))
      _v = PUBLIC;
    else if (!strcmp(s, "pri"))
      _v = PRIVATE;
    else if (!strcmp(s, "pro"))
      _v = PROTECTED;
    else
      success = false;
  }
  else {
    if (!strcmp(s, "public"))
      _v = PUBLIC;
    else if (!strcmp(s, "private"))
      _v = PRIVATE;
    else if (!strcmp(s, "protected"))
      _v = PROTECTED;
    else
      success = false;
  }
  return success;
}

const char *MVisibilityToken::Write() const
{
  static const char *names[] = {
    NULL,
    "public",
    "protected",
    "private"
  };

  return names[_v];
}

void MVisibility::LoadCtfx(xmlNodePtr in) 
{
  xmlChar *cscope;
  cscope = xmlGetProp(in, IS_UTF8 "scope");
  _t.Read(CAST_BAD cscope, true);
  free(cscope);
}

xmlNodePtr MVisibility::SaveUxf() const
{
  xmlNodePtr uvis;

  uvis = xmlNewNode(NULL, IS_UTF8 "Visibility");
  xmlNodeAddContent(uvis, BAD_CAST _t.Write());
  return uvis;
}

string DToS(int x) {
  char buf[30];
  snprintf(buf, 29, "%d", x);
  buf[29] = '\0';
  return string(buf);
}

string FToS(double x) {
  char buf[30];
  snprintf(buf, 29, "%g", x);
  buf[29] = '\0';
  return string(buf);
}

bool StringSplit(const string& s, const char *sep, string &before, string &after) {
  string::size_type pos = s.find(sep);
  before = s.substr(0, pos);
  if (pos == s.npos) {
    after = "";
    return false;
  }
  else {
    after = s.substr(pos + strlen(sep));
    return true;
  }
}

bool StringPop(const string& s, const char *sep, string &before, string &after) {
  string::size_type pos = s.find(sep);
  if (pos == s.npos) {
    // not found
    before = "";
    after = s;
    return false;
  }
  else {
    before = s.substr(0, pos);
    after = s.substr(pos + strlen(sep));
    return true;
  }
}

void myXPathRegisterDocNs(xmlXPathContextPtr ctx)
{
  ctx->namespaces = xmlGetNsList(ctx->doc, xmlDocGetRootElement(ctx->doc));
  ctx->nsNr = 0;
  if (ctx->namespaces != NULL) {
      while (ctx->namespaces[ctx->nsNr] != NULL)
	  ctx->nsNr++;
  }
}
