/* -*- C++ -*- */
// Medoosa: Support for Documentation of C++ Programs
// Master's Thesis, Faculty of Mathematics and Physics, Charles University
// Martin Vidner, 2001
// Released under the terms of the GNU General Public License.
/* $Id: common.h,v 1.20 2002/03/17 19:09:36 martin Exp $ */
#ifndef __M_COMMON_H
#define __M_COMMON_H

#include <list>
#include <set>
#include <string>
#include <libxml/xpath.h>
#include <libxml/tree.h>
#define CAST_BAD (char *)

/** A cast from a literal ASCII string to xmlChar* 
 * TODO: convert the casts to a parametrized macro
 */
#define IS_UTF8 (xmlChar *)

inline char PrevChar(list<string>::iterator i, list<string>::iterator begin) {
  assert (i != begin);
  return * ( (--i) -> c_str() );
}

/** If the token at i is a left bracket, skip a bracketed expression
    and stop after the right bracket or at end. */
void ForwardSkipBrackets(list<string>::iterator &i, char left, char right, list<string>::iterator end);

/** If the token before i is a right bracket, skip a bracketed expression
    and stop on the left bracket. An assert guards begin. */
void BackSkipBrackets(list<string>::iterator &i, char left, char right, list<string>::iterator begin);

/** finds ./tokens 
 * @returns NULL if no such node */
xmlNodePtr FindFirstTokenChild(xmlNodePtr node);

/** loads a list of tokens (tokens/tok/@value) */
list<string> LoadTokens(xmlNodePtr tokens);

/** constructs a UXF tree fragment TaggedValue/(Tag,Value) */
xmlNodePtr NewTaggedValue(xmlChar *tag, xmlChar *value);
/** Reads a UXF TaggedValue. If a parameter is NULL, it is not scanned */
void GetTaggedValue(xmlNodePtr tvnode, string *tag, string *value);
/** Reads a UXF TaggedValue. Returns the first node in Value */
xmlNodePtr GetTaggedNode(xmlNodePtr tvnode, string *tag);

/** concatenates a string list, putting spaces between words */
string Detokenize(const list<string> & sl);

/** @dia:pos -93,42 */
class MVisibilityToken {
private:
  enum V { 
    uninitialized,
    PUBLIC, 
    PROTECTED, 
    PRIVATE
  };
  V _v;
public:
  MVisibilityToken(): _v(uninitialized) {}; 
  /** Parses a string. 
   * @param tla If true, only three-letter abbreviation is parsed.
   * @returns True if parse successful.
   */
  bool Read(const char *s, bool tla = false);
  /** Stringifies.
   * @returns String or NULL if uninitialized.
   */
  const char *Write() const;
};

/** @dia:pos -83,33 */
class MVisibility {
public:
  void LoadCtfx(xmlNodePtr in);
  xmlNodePtr SaveUxf() const;
private:
  /** @dia:route 7,2;h,-67.656,36.6,-66,42,-72.3232 */
  MVisibilityToken _t;
};

string DToS(int x);
string FToS(double x);

/* StringSplit(foo, sep, head, foo) is safe */
/* StringSplit(foo, sep, foo, tail) is NOT safe */
/* If sep is not found, before=s, after="" */
/** @returns true if found */
bool StringSplit(const string&s, const char*sep, string&before, string&after);

/* StringPop(foo, sep, head, foo) is safe */
/* StringPop(foo, sep, foo, tail) is NOT safe */
/* If sep is not found, before="", after=s */
/** @returns true if found */
bool StringPop(const string&s, const char*sep, string &before, string &after);

/*
Register namespaces into a xpath context.
See http://mail.gnome.org/archives/xml/2001-July/msg00097.html
*/
void myXPathRegisterDocNs(xmlXPathContextPtr ctx);

#endif
