<?xml version="1.0"?>

<!-- $Id: dia-uxf.xsl,v 1.11 2002/03/17 19:08:27 martin Exp $ -->
<!--
// Medoosa: Support for Documentation of C++ Programs
// Master's Thesis, Faculty of Mathematics and Physics, Charles University
// Martin Vidner, 2001
// Released under the terms of the GNU General Public License.
-->
<!-- Dia to UML eXchange Format -->
<!-- Martin Vidner <mvidner@atlas.cz> -->

<xsl:stylesheet 
xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
xmlns:dia="http://www.lysator.liu.se/~alla/dia/"
xmlns:uxf="http://www.yy.cs.keio.ac.jp/~suzuki/project/uxf/"
version="1.0">

  <xsl:output indent="yes"/>
  <xsl:key name="objkey" match="dia:object" use="@id"/>

  <!-- disable copying of text and attrs -->
  <xsl:template match="text()|@*"/>
   
  <xsl:template match="/dia:diagram/dia:layer[1]">
    <uxf:UXF Version="2.0">
     
      <uxf:Model>
        <uxf:Name>A Model</uxf:Name>
	<xsl:apply-templates/>
      </uxf:Model>
    </uxf:UXF>
  </xsl:template>

<!-- ========================================================= -->

  <!-- Produces a string stripped of the first and last characters -->
  <!-- If the input is empty, it still produces a correct result. -->
  <!--
    Why did it not work ? <xsl:text>, duh!
  -->

  <xsl:template name="dia-string">
     <xsl:param name="s"/>
     <xsl:value-of select="substring($s, 2, string-length($s)-2)"/>
  </xsl:template>

<!-- ========================================================= -->

  <xsl:template match="dia:object[@type='UML - Class' and @version='0']">
    <uxf:Class>
      <uxf:TaggedValue>
        <uxf:Tag>DiagramPosition</uxf:Tag>
        <uxf:Value>
          <xsl:value-of select="dia:attribute[@name='elem_corner']/dia:point/@val"/>
        </uxf:Value>
      </uxf:TaggedValue>

        <xsl:variable name="class-name">
          <xsl:call-template name="dia-string">
            <xsl:with-param name="s" select="dia:attribute[@name='name']/dia:string"/>
          </xsl:call-template>
        </xsl:variable>
<!--
-->
      <uxf:Name>
	<xsl:value-of select="$class-name"/>
      </uxf:Name>

      <xsl:apply-templates/>

    </uxf:Class>
  </xsl:template>

<!-- ========================================================= -->

  <xsl:template match="dia:attribute[@name='attributes']">
    <xsl:for-each select="dia:composite[@type='umlattribute']">
      <uxf:Attribute>
        <xsl:variable name="attr-name-mul">
          <xsl:call-template name="dia-string">
            <xsl:with-param name="s" select="dia:attribute[@name='name']/dia:string"/>
          </xsl:call-template>
        </xsl:variable>
        <xsl:choose>
          <xsl:when test="contains($attr-name-mul,'[')">
            <uxf:Name>
              <xsl:value-of select="substring-before($attr-name-mul,'[')"/>
            </uxf:Name>
            <uxf:Multiplicity>
              <xsl:value-of select="substring-before(
			              substring-after($attr-name-mul,'['), ']'
			            )"/>
            </uxf:Multiplicity>
          </xsl:when>
          <xsl:otherwise>
            <uxf:Name><xsl:value-of select="$attr-name-mul"/></uxf:Name>
          </xsl:otherwise>
        </xsl:choose>


          <!-- TODO? ... -->
      </uxf:Attribute>
    </xsl:for-each>
  </xsl:template>

<!-- ========================================================= -->

  <xsl:template match="dia:object[@type='UML - Generalization' and @version='0']">
    <uxf:Generalization>
      <uxf:TaggedValue>
        <uxf:Tag>DiagramRoute</uxf:Tag>
        <uxf:Value><xsl:copy-of select="dia:attribute[@name='orth_points']"/></uxf:Value>
      </uxf:TaggedValue>

      <xsl:variable name="parent-n"
	    select="key('objkey',dia:connections/dia:connection[@handle='0']/@to)"/>
      <xsl:variable name="child-n"
	    select="key('objkey',dia:connections/dia:connection[@handle='1']/@to)"/>

      <uxf:Parent>
        <xsl:call-template name="dia-string">
          <xsl:with-param name="s" 
			  select="$parent-n/dia:attribute[@name='name']/dia:string"/>
        </xsl:call-template>
      </uxf:Parent>

      <uxf:Child>
        <xsl:call-template name="dia-string">
          <xsl:with-param name="s" 
			  select="$child-n/dia:attribute[@name='name']/dia:string"/>
        </xsl:call-template>
      </uxf:Child>

    </uxf:Generalization>
  </xsl:template>

<!-- ========================================================= -->

  <xsl:template match="dia:object[@type='UML - Association' and
		              @version='0' and
		              count(dia:connections/dia:connection) = 2 ]">
    <uxf:Association>
      <uxf:TaggedValue>
        <uxf:Tag>DiagramConnection1</uxf:Tag>
        <uxf:Value>
          <xsl:value-of select="dia:connections/
			         dia:connection[@handle='0']/
                                  @connection"/>
        </uxf:Value>
      </uxf:TaggedValue>
      <uxf:TaggedValue>
        <uxf:Tag>DiagramConnection2</uxf:Tag>
        <uxf:Value>
          <xsl:value-of select="dia:connections/
			         dia:connection[@handle='1']/
                                  @connection"/>
        </uxf:Value>
      </uxf:TaggedValue>
      <uxf:TaggedValue>
        <uxf:Tag>DiagramRoute</uxf:Tag>
        <uxf:Value><xsl:copy-of select="dia:attribute[@name='orth_points']"/></uxf:Value>
      </uxf:TaggedValue>

      <xsl:for-each select="dia:attribute[@name='ends']/dia:composite">
        <uxf:AssociationEnd>
          <xsl:variable name="end-num" select="position() - 1"/>
          <xsl:variable name="type-key" select="../../dia:connections/
			                        dia:connection[@handle=$end-num]/
			                        @to"/>
          <xsl:variable name="type-n" select="key('objkey', $type-key)"/>
          <uxf:Type>
            <xsl:call-template name="dia-string">
              <xsl:with-param name="s" 
			      select="$type-n/dia:attribute[@name='name']/dia:string"/>
            </xsl:call-template>
          </uxf:Type>

          <xsl:variable name="visi-name">
            <xsl:call-template name="dia-string">
              <xsl:with-param name="s" 
			      select="dia:attribute[@name='role']/dia:string"/>
            </xsl:call-template>
          </xsl:variable>
          <xsl:variable name="visibility">
            <xsl:choose>
              <xsl:when test="starts-with($visi-name, '+')">public</xsl:when>
              <xsl:when test="starts-with($visi-name, '-')">private</xsl:when>
              <xsl:when test="starts-with($visi-name, '#')">protected</xsl:when>
            </xsl:choose>
          </xsl:variable>

          <uxf:Name>
            <xsl:choose>
              <xsl:when test="string($visibility)">
                <xsl:value-of select="substring($visi-name, 2)"/>
              </xsl:when>
              <xsl:otherwise> 
                <xsl:value-of select="$visi-name"/>
              </xsl:otherwise>
            </xsl:choose>
          </uxf:Name>

          <!-- Qualifier unsupported by Dia -->

          <xsl:variable name="aggr" 
			select="dia:attribute[@name='aggregate']/dia:enum/@val"/>
          <xsl:if test="$aggr != 0">
            <uxf:AggregationKind>
              <xsl:choose>
                <xsl:when test="$aggr = 1">AggregationWhole</xsl:when>
                <xsl:otherwise>CompositionWhole</xsl:otherwise>
              </xsl:choose>
            </uxf:AggregationKind>
          </xsl:if>

          <xsl:variable name="mult-order">
            <xsl:call-template name="dia-string">
              <xsl:with-param name="s" 
			      select="dia:attribute[@name='multiplicity']/dia:string"/>
            </xsl:call-template>
          </xsl:variable>
	  <xsl:choose>
	    <xsl:when test="contains($mult-order,'{')">
	      <uxf:Multiplicity>
		<xsl:value-of select="normalize-space(
			                substring-before($mult-order,'{')
                                      )"/>
	      </uxf:Multiplicity>
	      <uxf:Ordering>
		<xsl:value-of select="substring-before(
				        substring-after($mult-order,'{'), '}'
				      )"/>
	      </uxf:Ordering>
	    </xsl:when>
	    <xsl:otherwise>
	      <uxf:Multiplicity>
		<xsl:value-of select="$mult-order"/>
	      </uxf:Multiplicity>
	    </xsl:otherwise>
	  </xsl:choose>

          <!-- isNavigable dropped, induced from existence of role names -->

          <!-- Visibility and TargetScope not Diffable -->
          <xsl:if test="string($visibility)">
            <uxf:Visibility>
              <xsl:value-of select="$visibility"/>
            </uxf:Visibility>
          </xsl:if>

          <!-- TODO (Changeability), Ordering... -->
        </uxf:AssociationEnd>
      </xsl:for-each>
    </uxf:Association>
  </xsl:template>

  <xsl:template match="dia:object[@type='UML - Association' and
		              @version='0' and
		              count(dia:connections/dia:connection) != 2 ]">
    <xsl:message>
      <xsl:text>Warning, unconnected association (roles </xsl:text>
      <xsl:value-of select="dia:attribute[@name='ends']/dia:composite[1]/
		              dia:attribute[@name='role']/dia:string"/>
      <xsl:text> and </xsl:text>
      <xsl:value-of select="dia:attribute[@name='ends']/dia:composite[2]/
		              dia:attribute[@name='role']/dia:string"/>
      <xsl:text>)</xsl:text>
    </xsl:message>
  </xsl:template>
<!-- ========================================================= -->

<!--
-->
</xsl:stylesheet>
