// -*- C++ -*-
/* $Id: diagram.h,v 1.13 2002/03/17 19:09:36 martin Exp $ */
// Medoosa: Support for Documentation of C++ Programs
// Master's Thesis, Faculty of Mathematics and Physics, Charles University
// Martin Vidner, 2001
// Released under the terms of the GNU General Public License.
#ifndef __M_DIAGRAM_H
#define __M_DIAGRAM_H

#include <list>
#include <string>
#include <libxml/tree.h>

/** XML namespace used by SaveDia methods */
extern xmlNsPtr diaNS;

/** @dia:pos 5,75 */
class MDiagramPoint {
 public:
  MDiagramPoint();
  MDiagramPoint(xmlNodePtr dnode);
  void LoadDia(xmlNodePtr dnode);
  void LoadString(const char *s);
  xmlNodePtr SaveDia() const;
  string SaveString() const;
  bool operator == (const MDiagramPoint &p) const;
  double _x, _y;
  /** @returns true if the point was default constructed */
  bool IsUnspecified() const {return _isUnspecified;}
  bool _isUnspecified;
};

/** Orthogonal! */
/** @dia:pos -32,73 */
class MDiagramRoute {
 public:
  /** @param dnode A XML element whose children are dia:point elements */
  void LoadDia(xmlNodePtr dnode);
  /** @param s "h,3.5,4.5,7,2,5" see [TODO] for explanation */
  void LoadString(const char *s);
  /** Makes orth_points and orth_orient, embedded in a TaggedValue. 
      Or an empty node, if uninitialized. */
  xmlNodePtr SaveDia() const;
  string SaveString() const;

  bool operator == (const MDiagramRoute &b) const {
    return _points == b._points;
  }

  /** @dia:route 0,2;h,5,75,1,73,-6.96 */
  /** @uml:order ordered */
  list<MDiagramPoint> _points;
};

/** @dia:pos -31,87 */
/** @dia:route MDiagramRoute;v,-19.48,79,83,-20.6616,87 */
class MAssociationRoute : public MDiagramRoute {
public:
  MAssociationRoute();
  /** @param s "c1,c2;route", two connection points and a
      DiagramRoute, which see */
  void LoadString(const string &s);
  string SaveString() const;
  // this functionality is performed by MAssociation::LoadUxf
  //  void LoadDia(xmlNodePtr dnode); 
  void SaveDia(xmlNodePtr uassoc) const;

  int _connectionPoints[2];
};

#endif
