/* $Id: diff-uxf.cc,v 1.9 2001/04/23 16:10:29 martin Exp $ */
// Medoosa: Support for Documentation of C++ Programs
// Master's Thesis, Faculty of Mathematics and Physics, Charles University
// Martin Vidner, 2001
// Released under the terms of the GNU General Public License.
#include <stdlib.h>
#include <libxml/parser.h>
#include <libxml/tree.h>

#include "model.h"
#include "common.h"
#include "debug.h"

xmlDocPtr compareUxf(xmlDocPtr oldDoc, xmlDocPtr newDoc) {
  xmlDocPtr patchDoc = xmlNewDoc(IS_UTF8 "1.0");
  patchDoc->children = (xmlNodePtr) xmlNewDtd(patchDoc, IS_UTF8 "patch", 
					      NULL, IS_UTF8 "patch.dtd");

  xmlNodePtr patchRoot = xmlNewDocNode(patchDoc, NULL, IS_UTF8 "patch", NULL);
  xmlDocSetRootElement(patchDoc, patchRoot);

  MModel oldClasses;
  MModel newClasses;
  oldClasses.LoadUxf(oldDoc);
  newClasses.LoadUxf(newDoc);

  newClasses.Diff(oldClasses, patchRoot);

  return patchDoc;
}

int main(int argc, char **argv) {
  xmlDocPtr olddoc, newdoc, patchdoc;

  if (getenv("MEDOOSA_DEBUG_ALL") != NULL)
    mDebug.EnableAll();
  else
    mDebug.Enable(getenv("MEDOOSA_DEBUG"));

  if (argc != 3) {
    fprintf(stderr, "usage: %s before-dia.uxf after-dia.uxf\n", argv[0]);
    exit(1);
  }
     
  LIBXML_TEST_VERSION;
  olddoc = xmlParseFile(argv[1]);
  if (olddoc == NULL) {
    fprintf(stderr, "error parsing %s\n", argv[1]);
    exit(1);
  }
  newdoc = xmlParseFile(argv[2]);
  if (newdoc == NULL) {
    fprintf(stderr, "error parsing %s\n", argv[2]);
    exit(1);
  }

  patchdoc = compareUxf(olddoc, newdoc);
  int result;
  result = xmlSaveFile("-", patchdoc);

  xmlFreeDoc(patchdoc);
  xmlFreeDoc(newdoc);  
  xmlFreeDoc(olddoc);
  return result == -1 ? 1 : 0;
}
