// $Id: element.cc,v 1.13 2001/04/23 16:10:29 martin Exp $
// Medoosa: Support for Documentation of C++ Programs
// Master's Thesis, Faculty of Mathematics and Physics, Charles University
// Martin Vidner, 2001
// Released under the terms of the GNU General Public License.
#include "element.h"
#include "namespace.h"
#include "common.h"
#include "class.h"
#include "debug.h"
#include <libxml/parser.h>
#include <iostream>
#include <typeinfo>

void MSourceInfo::LoadCtfx(xmlNodePtr in) {
  xmlChar *line = xmlGetProp(in, IS_UTF8 "line");
  _sourceLine = atoi(CAST_BAD line);
  free(line);
}

void MSourceInfo::SaveUxfTo(xmlNodePtr parent) const {
  xmlAddChild(parent, NewTaggedValue(IS_UTF8 "SourceLine", 
				     BAD_CAST DToS(_sourceLine).c_str()));  
}

void MSourceInfo::SaveDiffTo(xmlNodePtr parent) const {
  xmlNewChild(parent, NULL, 
	      IS_UTF8 "line", BAD_CAST DToS(_sourceLine).c_str());
}

// ---------------------------------------------------------------

MModelElement::MModelElement(string name, MNamespace *owner) :
  _name(name),
  _owner(owner)
{
}

/*
xmlNodePtr MModelElement::SaveUxf() const {
  xmlNodePtr node = xmlNewNode(NULL, UxfName());
  xmlNewChild(node, NULL, IS_UTF8 "Name", BAD_CAST _name.c_str()); 
  return node;
}
*/

void MModelElement::SetOwner(MNamespace *owner) {
  _owner = owner;
}

string MModelElement::GetQualifiedName() const { 
  string qname;
  if (_owner != NULL) {
    qname = _owner->GetQualifiedName();
    if (!qname.empty())
      qname += "::";
  }
  qname += _name;
  return qname; 
}

void MModelElement::LoadComments(xmlNodePtr cnode) {
  //  xmlNodePtr parent;
  xmlNodePtr sibling;
  string comment;
  assert(cnode->parent != NULL);

  for (sibling = cnode->prev; sibling != NULL; sibling = sibling->prev) {
    if (sibling->type == XML_ELEMENT_NODE) {
      // a directive:
      if (!xmlStrcmp(sibling->name, IS_UTF8 "cmd")) {
	// the token list has a single member
	comment = * LoadTokens(FindFirstTokenChild(sibling)).begin();
	string dname(comment, 0, comment.find_first_of(" \t\n"));
	string dvalue;
	size_t valuePos = comment.find_first_not_of(" \t\n",dname.size());
	if (valuePos != string::npos)
	  dvalue = comment.substr(valuePos);

	if (mDebug.IsEnabled('@'))
	  cerr << '"' << dname << "\"->\"" << dvalue << '"' << endl;
	_directives.insert(Directives::value_type(dname, dvalue));
      }
      // brief comment:
      else if (!xmlStrcmp(sibling->name, IS_UTF8 "cmb")) {
	// TODO not interested, ccdoc does the job
      }
      // full comment:
      else if (!xmlStrcmp(sibling->name, IS_UTF8 "cmf")) {
	// TODO not interested, ccdoc does the job
      }
      else
	break; // we are past the contiguous block of comments
    }
  }
}

string MModelElement::SourceFile() const {
  return dynamic_cast<const MClass*>(_owner)->SourceFile();
}

xmlNodePtr MModelElement::DiffRequestChange(xmlNodePtr patchRoot, 
					    const string &directive, 
					    const string &value) const {
  xmlNodePtr patchNode;

  patchNode = xmlNewChild(patchRoot, NULL, IS_UTF8 "request", NULL);

  // SourceFile only works for elements in a class, TODO that's broken
  string sourceFile = SourceFile();
  xmlNewChild(patchNode, NULL, IS_UTF8 "file", BAD_CAST sourceFile.c_str());

  _sourceInfo.SaveDiffTo(patchNode);

  string fullName, ns, shortName;
  shortName = fullName = GetQualifiedName();
  // discard namespace prefixes
  while (StringPop(shortName, "::", ns, shortName))
    ;
  // add name information to check for misplaced patches
  xmlNewChild(patchNode, NULL, IS_UTF8 "full", BAD_CAST fullName.c_str());
  xmlNewChild(patchNode, NULL, IS_UTF8 "short", BAD_CAST shortName.c_str());

  xmlNewChild(patchNode, NULL,
	      IS_UTF8 "directive", BAD_CAST directive.c_str());
  xmlNewChild(patchNode, NULL,
	      IS_UTF8 "value", BAD_CAST value.c_str());

  return patchNode;
}
