// -*- C++ -*-
/* $Id: element.h,v 1.12 2001/04/23 16:10:29 martin Exp $ */
// Medoosa: Support for Documentation of C++ Programs
// Master's Thesis, Faculty of Mathematics and Physics, Charles University
// Martin Vidner, 2001
// Released under the terms of the GNU General Public License.
#ifndef __M_ELEMENT_H
#define __M_ELEMENT_H

#include <string>
#include <map>
//#include <list>
//#include <utility>
#include <libxml/tree.h>

class MNamespace;

/** Keeps track where a model element comes from. */
/** @dia:pos -59,7 */
struct MSourceInfo {
  MSourceInfo() : _sourceLine(-1) {}
  int _sourceLine;
  /** Line is taken from in/@line */
  void LoadCtfx(xmlNodePtr in);
  /** writes a tagged value */
  void SaveUxfTo(xmlNodePtr parent) const;
  /** writes line */
  void SaveDiffTo(xmlNodePtr parent) const;
};

/** @dia:pos -29,4 */
class MModelElement {
public:
  /** Sets the owner.
   *
   * When a MNamespace is copied, its children get notified of the
   * new owner.
   */
  void SetOwner(MNamespace *owner);

  /** Need a virtual destructor. */
  virtual ~MModelElement() {}
  /** eg, MClass::Clone() const {return new MClass(*this);} */
  virtual MModelElement * Clone() const = 0;

  /** Makes a UXF representation 
   * @param ownerUxf The UXF node of my owner, that is where I put myself...
   * @param modelUxf ... unless I am a MNamespace and therefore 
   *                 cannot nest in UXF and so I put myself in modelUxf
   */
  virtual void SaveUxf(xmlNodePtr ownerUxf, xmlNodePtr modelUxf) const = 0;
  //  virtual void LoadParseTree(xmlNodePtr cnode) = 0;
  virtual void LoadUxf(xmlNodePtr unode) = 0;
  virtual void Diff(const MModelElement &old, xmlNodePtr patchRoot) const = 0;
  // perhaps called by Analyze?
  virtual void ProcessDirectives() = 0;
  virtual string SourceFile() const;
  /** @param Attach a directive to this element's source */
  xmlNodePtr DiffRequestChange(xmlNodePtr patchRoot, const string &directive, const string &value) const;
  
  string GetName() const { return _name; }
  string GetQualifiedName() const;
protected:
  MModelElement(string name, MNamespace *owner);

  // default copy ctor and assignment are ok

  /** Loads documentation comments and @-directives.
   *
   * @param cnode The CTFX node of this element.
   */
  void LoadComments(xmlNodePtr cnode);
  //  virtual xmlChar *UxfName() const = 0;

  /** The "short" name.
   *
   * May be empty for some elements */
  string _name;

  /** This element's containing namespace or class.
   *
   * Is NULL only for the root namespace.
   */
  /** @dia:route 21,30;h,-5.4144,10.9,2,33.9,12 */
  /** @uml:other _children,MNamespace */
  /** @uml:mul 0..1 */
  MNamespace *_owner;
  /** Just the source line, but with convenience methods */
  /** @dia:route 0,2;h,-29,4,-31,7,-36.8688 */
  MSourceInfo _sourceInfo;

  // it could also be a multimap but that wouldn't preserve the order
  // in the file.
  //typedef list< pair<string, string> > Directives;
  typedef multimap<string, string> Directives;
  Directives _directives;
private:
};

#endif
