// $Id: model.cc,v 1.9 2002/03/17 19:09:36 martin Exp $
// Medoosa: Support for Documentation of C++ Programs
// Master's Thesis, Faculty of Mathematics and Physics, Charles University
// Martin Vidner, 2001
// Released under the terms of the GNU General Public License.
#include "model.h"

#include <libxml/xpath.h>
#include <list>
#include "element.h"
#include "common.h"
#include "class.h"
#include "attribute.h"

void MModel::LoadParseTree(xmlNodePtr cnode) {
  xmlNodePtr cchild;

  for (cchild = cnode->children; cchild != NULL; cchild = cchild->next) {
    if (cchild->type == XML_ELEMENT_NODE) {

      if (!xmlStrcmp(cchild->name, IS_UTF8 "fil")) {
	list<string> toks = LoadTokens(FindFirstTokenChild(cchild));
	assert(toks.size() == 3);
	string fileName = *toks.begin();
	string projectName = *toks.rbegin();
	
	if (!_isNameValid) {
	  _name = projectName;
	}
	else {
	  if (projectName != _name) {
	    //warning
	    cerr << "File " << fileName 
		 << " specifies a new project name " << projectName
		 << ". Ignored.\n";
	  }
	}

	_rootNs.LoadParseTree(cchild, fileName);
      }
      else {
	cerr << "Unknown element " << (char*)cchild->name << ".\n";
      }
    }
  }

  _rootNs.ProcessDirectives();
}



xmlDocPtr MModel::CreateUxf() {
  xmlDocPtr modelDoc = xmlNewDoc(IS_UTF8 "1.0");
  modelDoc->children = (xmlNodePtr) xmlNewDtd(modelDoc, IS_UTF8 "UXF", 
					      NULL, IS_UTF8 "uml.dtd");
  xmlNodePtr root = xmlNewDocNode(modelDoc, NULL, IS_UTF8 "UXF", NULL);
  xmlDocSetRootElement(modelDoc, root);
  xmlSetProp(root, IS_UTF8 "Version", IS_UTF8 "2.0");
  diaNS = xmlNewNs(root, IS_UTF8 "http://www.lysator.liu.se/~alla/dia/", IS_UTF8 "dia");
  xmlNewNs(root, IS_UTF8 "http://www.yy.cs.keio.ac.jp/~suzuki/project/uxf/", NULL);
  xmlNewNs(root, IS_UTF8 "http://www.yy.cs.keio.ac.jp/~suzuki/project/uxf/", IS_UTF8 "uxf");

  xmlNodePtr mdl = xmlNewChild(root, NULL, IS_UTF8 "Model", NULL);
  xmlNewChild(mdl, NULL, IS_UTF8 "Name", BAD_CAST _name.c_str());

  _rootNs.SaveUxf(mdl, mdl);

  return modelDoc;
}

void MModel::LoadUxf(xmlDocPtr modelDoc) {
  xmlXPathContextPtr ctx;
  xmlXPathObjectPtr obj;
  xmlNodeSetPtr nodes;
  xmlChar *name;

  ctx = xmlXPathNewContext(modelDoc);
  myXPathRegisterDocNs(ctx);

  obj = xmlXPathEval(IS_UTF8 "string(/uxf:UXF/uxf:Model/uxf:Name)", ctx);
  name = obj->stringval;
  _name = CAST_BAD name;
  xmlXPathFreeObject(obj);

  obj = xmlXPathEval(IS_UTF8 "/uxf:UXF/uxf:Model", ctx);
  nodes = obj->nodesetval;
  assert(nodes->nodeNr == 1);
  _rootNs.LoadUxf(nodes->nodeTab[0]);
  xmlXPathFreeObject(obj);

  obj = xmlXPathEval(IS_UTF8 "/uxf:UXF/uxf:Model/uxf:Association/uxf:AssociationEnd", ctx);
  nodes = obj->nodesetval;
  if (nodes != NULL) {
   for (int i = 0; i < nodes->nodeNr; ++i) {
    xmlNodePtr uAssocEnd = nodes->nodeTab[i];

    xmlXPathContextPtr aectx;
    xmlXPathObjectPtr typeobj;
    string className;
    MClass *aclass;

    aectx = xmlXPathNewContext(modelDoc);
    myXPathRegisterDocNs(aectx);

    aectx->node = uAssocEnd;
    typeobj = xmlXPathEval(IS_UTF8 "string(uxf:Type)", aectx);
    className = CAST_BAD typeobj->stringval;
    aclass = dynamic_cast<MClass*>(_rootNs.GetElement(className));
    if (aclass == NULL)
      cerr << "Error, " << className << " is not a class.\n";
    else
      aclass->LoadAssocAttrUxf(uAssocEnd);

    xmlXPathFreeObject(typeobj);
    xmlXPathFreeContext(aectx);
   }
  }
  xmlXPathFreeObject(obj);

  xmlXPathFreeContext(ctx);
}

void MModel::Diff(const MModel &oldModel, xmlNodePtr patchRoot) {
  _rootNs.Diff(oldModel._rootNs, patchRoot);
}

void MModel::InsertStlClassStubs() {
  // TODO
}
