// -*- C++ -*-
/* $Id: namespace.h,v 1.10 2001/04/23 16:10:30 martin Exp $ */
// Medoosa: Support for Documentation of C++ Programs
// Master's Thesis, Faculty of Mathematics and Physics, Charles University
// Martin Vidner, 2001
// Released under the terms of the GNU General Public License.
#ifndef __M_NAMESPACE_H
#define __M_NAMESPACE_H

#include "element.h"
#include <map>
#include <set>
#include <string>
#include <libxml/tree.h>

class MClass;

/** @dia:pos 12,23 */
/** @dia:route MModelElement;v,-17.2072,17.8,20,26.944,23 */
class MNamespace : public MModelElement {
public:
  MNamespace(string name, MNamespace *owner): MModelElement(name, owner) {}

  virtual ~MNamespace();

  /** Copy constructor.
      Must SetOwner for all children. */
  MNamespace(const MNamespace &);

  /** Assignment operator.
      Must SetOwner for all children. */
  MNamespace & operator = (const MNamespace &);

  virtual void SaveUxf(xmlNodePtr ownerUxf, xmlNodePtr modelUxf) const;
  virtual void Diff(const MModelElement &old, xmlNodePtr patchRoot) const;
  virtual void LoadParseTree(xmlNodePtr cnode, const string &fileName) = 0;
  virtual void ProcessDirectives();

  /** Finds an element by (possibly qualified) name by applying the
      proper scoping rules */
  MModelElement * GetElement(const string &) const;

  //  MNamespace * GetRoot() const;
protected:
  /** Used by LoadUxf because it reads qualified class names 
   *
   * Creates a class in a proper namespace, creating namespaces if
   * they do not exist.
   * @returns The newly created class*/
  MClass* InsertQualifiedClass(const string& qname);

  /** It is a multimap because of operation overloading. */
  /** @uml:other _owner,MModelElement */
  /** @uml:mul * */
  /** @uml:order ordered */
  multimap<string, MModelElement *> _children;
  typedef multimap<string, MModelElement *> ChildrenMap;
  /** a class is declared in a single file, a namespace may be in more */
  set<string> _containingFiles;
private:
  /** Destroys all children and clears the map */
  void DeleteChildren();
  void CopyChildren(const MNamespace &);
  /** Called by GetElement, searches owning namespaces and (not yet)
      base clases */
  MModelElement * GetElementElsewhere(const string &) const;
};

/** @dia:pos 21,40 */
/** @dia:route MNamespace;v,26.944,36,38,38.1256,40 */
class MCxxNamespace : public MNamespace {
public:
  MCxxNamespace(const string& name, MNamespace *owner): MNamespace(name, owner) {}
  virtual ~MCxxNamespace();
  MCxxNamespace(const MCxxNamespace & b): MNamespace(b) {}
  
  static string LoadName(xmlNodePtr cnode);

  virtual MModelElement * Clone() const {return new MCxxNamespace(*this);}
  virtual void LoadParseTree(xmlNodePtr cnode, const string &fileName);
  virtual void LoadUxf(xmlNodePtr uclass);

private:
  bool IsRoot() { return _owner == NULL; }

  //prohibitions:
private:
  MCxxNamespace & operator = (const MCxxNamespace &);
};

#endif
