/* $Id: operation.cc,v 1.14 2001/04/23 16:10:30 martin Exp $ */
// Medoosa: Support for Documentation of C++ Programs
// Master's Thesis, Faculty of Mathematics and Physics, Charles University
// Martin Vidner, 2001
// Released under the terms of the GNU General Public License.
#include "operation.h"
#include <stdlib.h>
#include <libxml/parser.h>
#include <libxml/tree.h>
#include "class.h"

void MOperation::LoadParseTree(xmlNodePtr coper) {
  // name
  xmlChar *cname;
  cname = xmlGetProp(coper, IS_UTF8 "name");
  if (!xmlStrcmp(coper->name, IS_UTF8 "cld"))
    _name += "~";
  _name += CAST_BAD cname;
  free(cname);

  // type 
  list<string> tokens;
  tokens = LoadTokens(FindFirstTokenChild(coper));
  // Now, tokens is initialized.

  list<string> dummyname, type, value, params;
  MClass *cls = dynamic_cast<MClass*>(_owner);
  cls->ParseDeclaration(tokens.begin(), tokens.end(),
			   dummyname, type, value, params);

  ParseProperties(type);
  _type.Load(type);

  _isAbstract = !value.empty();
  if (_isAbstract)
    cls->SetAbstract();

  // parameters
  list<string>::iterator paramsBegin = params.begin();
  list<string>::iterator paramsEnd = params.end();

  // Common part for function and template parameters, '>', ')'
  list<string>::iterator i = paramsBegin;
  ++paramsBegin; // into the brackets
  for (i = paramsBegin; i != paramsEnd && * i->c_str() != ')'; ) {
    // set [parambegin,i) range 
    list<string>::iterator paramBegin = i;
    char c;
    while (i != paramsEnd, c = * i->c_str(), c != ',' && c != ')') {
      if (c == '(')
	ForwardSkipBrackets(i, '(', ')', paramsEnd);
      else if (c == '<')
	ForwardSkipBrackets(i, '<', '>', paramsEnd);
      else
	++i;
    }
    _parameters.push_back(MParameter(paramBegin, i, cls));
    // skip ','
    //if (i != paramsEnd)
    if (* i->c_str() != ')')
      ++i;
  }
  // End common part 
  _visibility.LoadCtfx(coper);
}

void MOperation::SaveUxf(xmlNodePtr ownerUxf, xmlNodePtr /*modelUxf*/) const {
  xmlNodePtr uoper;
  uoper = xmlNewNode(NULL, IS_UTF8 "Operation");
  
  // (this is a UXF extension), or rather a UXF deficiency
  if (_isQuery) 
    xmlAddChild(uoper, NewTaggedValue(IS_UTF8 "IsQuery", IS_UTF8 "true"));

  xmlNewChild(uoper, NULL, IS_UTF8 "Name", BAD_CAST _name.c_str());
  xmlAddChild(uoper, _visibility.SaveUxf());
  xmlAddChild(uoper, _type.SaveUxf(IS_UTF8 "ReturnType"));
  if (_isAbstract)
    xmlNewChild(uoper, NULL, IS_UTF8 "IsAbstract", IS_UTF8 "true");
  if (_ownerScope == CLASSIFIER)
    xmlNewChild(uoper, NULL, IS_UTF8 "OwnerScope", IS_UTF8 "classifier");

  {
    list<MParameter>::const_iterator i;
    for (i = _parameters.begin(); i != _parameters.end(); ++i)
      xmlAddChild(uoper, i->SaveUxf());
  }

  xmlAddChild(ownerUxf, uoper);
}

void MOperation::LoadUxf(xmlNodePtr) {
  // Do nothing.
  // MClass::LoadUxf never calls us...
}

void MOperation::Diff(const MModelElement &, xmlNodePtr) const {
  // ... and that's because the diff is irrelevant --
  // there are no hints to be taken from the user about operations.
  // Do nothing.
}

void MOperation::ProcessDirectives() {
  // no directives to process yet
#if 0
  // this is a generic skeleton
  Directives::iterator i;
  for (i = _directives.begin(); i != _directives.end(); ++i) {
    string dname = i->first;
    string dvalue = i->second;

    if (dname == "@...") {
      //TODO...
    } // if @...
    //else if TODO...
  } // for all directives
#endif
}

void MOperation::ParseProperties(list<string> &type) {
  // this should actually be in ParseDeclaration because
  // it can be confused by "operator char * const();"
  // but that should be rare.

  MFeature::ParseProperties(type);

  list<string>::iterator i = type.end();
  if (i != type.begin()) {
    --i;
    const char * s = i->c_str();

    if (!strcmp(s, "const")) {
      _isQuery = true;
      type.erase(i);
    }
  }

  //TODO: virtual -> !isLeaf
}
