// -*- C++ -*-
// $Id: traits.h,v 1.6 2001/04/23 16:10:30 martin Exp $
// Medoosa: Support for Documentation of C++ Programs
// Master's Thesis, Faculty of Mathematics and Physics, Charles University
// Martin Vidner, 2001
// Released under the terms of the GNU General Public License.
#ifndef __M_TRAITS_H
#define __M_TRAITS_H

#include "umltypes.h"

// ---------------------------------------------------------------
// --- Implementors ---

/* The *Traits structures are used to describe template classes that
implement various UML constructs. For example, STL map implements
qualified associations. Medoosa has a default set of container traits
for the STL. The user can supply traits for his own implementations
using @-directives. */

/** @dia:pos -155,5 */
class MClassTraits {
public:
  /** Will not appear as a class box in the model */
  bool _isSuppressed;
};

/** @dia:pos -158,11 */
/** @dia:route MClassTraits;v,-149.752,7.8,10,-151.298,11 */
class MPointerTraits : public MClassTraits {
public:
  /** Which of the template parameters is the target type */
  int _targetTypeParameter;
};

// overly complicated, anyway useful only for poorly designed
// containers (see argument in Ruminationson C++)?
/** Does the container hold objects of the target type (strong, composition)
 * or pointers to objects of the target type (weak, association)?
((ContainerKind is C++ counterpart of UML AggregationKind)) */
enum MContainerKind {
  //  NONE,
  OBJECTS, // implies AK::COMPOSITE for the attribute
  POINTERS // AK::NONE
};

/** Characterizes container classes.
 *
 * See its use for STL containers. For example:
 * list is unqualified, *, ordered; target is the first (and only) parameter.
 * map is qualified by the first parameter, 0..1, sorted; target is the second.
 */
/** @dia:pos -158,17 */
/** @dia:route MPointerTraits;v,-151.298,13.8,16,-150.57,17 */
class MContainerTraits : public MPointerTraits {
public:
  // MContainerKind _containerKind;
  /** Are the contained objects qualified */
  bool _isQualified;
  /** Which of the template parameters is the qualifier */
  int _qualifierTypeParameter;
  /** How many (qualified) objects it can contain */
  /** @dia:route 7,0;h,-143.141,21.4,-99,52,-95 */
  MMultiplicity _multiplicity;
  /** How the contained objects are ordered */
  MOrderingKind _ordering;
};

#endif
