// $Id: umltypes.cc,v 1.5 2001/04/23 16:10:30 martin Exp $
// Medoosa: Support for Documentation of C++ Programs
// Master's Thesis, Faculty of Mathematics and Physics, Charles University
// Martin Vidner, 2001
// Released under the terms of the GNU General Public License.
#include "umltypes.h"
#include <climits>
#include "common.h"

bool MMultiplicity::operator == (const MMultiplicity& b) const {
  if (_isUnspecified)
    return b._isUnspecified;
  else
    return !b._isUnspecified && _lower == b._lower && _upper == b._upper;
}

void MMultiplicity::Compose(const MMultiplicity& b) {
  if (!_isUnspecified) {
    if (b._isUnspecified)
      _isUnspecified = true;
    else {
      _lower *= b._lower;
      if (_upper == UINT_MAX || b._upper == UINT_MAX)
	_upper = UINT_MAX;
      else
	_upper *= b._upper;
    }
  }
}

void MMultiplicity::LoadString(const string &s) {
  bool single;
  string lower, upper;
  single = ! StringPop(s, "..", lower, upper);
  if (single) {
    if (upper.empty()) {
      // empty string
      _isUnspecified = true;
    }
    else {
      // "*" or "number"
      if (upper == "*") {
	_upper = UINT_MAX;
	_lower = 0;
      }
      else {
	_upper = atoi(upper.c_str());
	_lower = _upper;
      }
      _isUnspecified = false;
    } 
  }
  else {
    // "number..number" or "number..*"
    _lower = atoi(lower.c_str());
    if (upper == "*")
      _upper = UINT_MAX;
    else 
      _upper = atoi(upper.c_str());

    _isUnspecified = false;
  }
}

string MMultiplicity::SaveString() const {
  assert(_isUnspecified || _lower <= _upper);
  string result;
  if (_isUnspecified)
    result = "";
  else if (_lower == 0 && _upper == UINT_MAX)
    result = "*";
  else {
    result = DToS(_lower);
    if (_lower != _upper) {
      result += "..";
      result += (_upper == UINT_MAX)? "*" : DToS(_upper);
    }
  }
  return result;
}

void MMultiplicity::LoadUxf(xmlNodePtr umul) {
  char *s = CAST_BAD xmlNodeGetContent(umul);
  LoadString(s);
  free(s);
}

xmlNodePtr MMultiplicity::SaveUxf() const {
  xmlNodePtr umul;
  if (_isUnspecified) 
    umul = xmlNewText(IS_UTF8 "");
  else {
    umul = xmlNewNode(NULL, IS_UTF8 "Multiplicity");
    xmlAddChild(umul, xmlNewText(BAD_CAST SaveString().c_str()));
  }
  return umul;
}

