<?xml version="1.0"?>

<!-- $Id: uxf-dia.xsl,v 1.15 2002/03/17 19:08:27 martin Exp $ -->
<!--
// Medoosa: Support for Documentation of C++ Programs
// Master's Thesis, Faculty of Mathematics and Physics, Charles University
// Martin Vidner, 2001
// Released under the terms of the GNU General Public License.
-->
<!-- Draw UXF class diagrams using Dia. See respective URLs below -->
<!-- Martin Vidner <mvidner@atlas.cz> -->

<xsl:stylesheet 
xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
xmlns:dia="http://www.lysator.liu.se/~alla/dia/"
xmlns:uxf="http://www.yy.cs.keio.ac.jp/~suzuki/project/uxf/"
version="1.0">

  <xsl:param name="max-param-len" select="80"/>
  <xsl:param name="members" select="1"/>
  <xsl:param name="attr-multiplicities" select="0"/>

  <xsl:output indent="yes"/>
  <xsl:key name="classkey" match="uxf:Class" use="uxf:Name"/>

  <!-- disable copying of text and attrs -->
  <xsl:template match="text()|@*"/>
   
<!-- ========================================================= -->

<!--  <xsl:template match="//ClassDiagram"> -->
  <xsl:template match="/uxf:UXF">
    <dia:diagram>
      <dia:diagramdata>
	<dia:attribute name="background">
	  <dia:color val="#ffffff"/>
	</dia:attribute>
	<dia:attribute name="paper">
	  <dia:composite type="paper">
	    <dia:attribute name="name">
	      <dia:string>#A4#</dia:string>
	    </dia:attribute>
	    <dia:attribute name="tmargin">
	      <dia:real val="2.82"/>
	    </dia:attribute>
	    <dia:attribute name="bmargin">
	      <dia:real val="2.82"/>
	    </dia:attribute>
	    <dia:attribute name="lmargin">
	      <dia:real val="2.82"/>
	    </dia:attribute>
	    <dia:attribute name="rmargin">
	      <dia:real val="2.82"/>
	    </dia:attribute>
	    <dia:attribute name="is_portrait">
	      <dia:boolean val="true"/>
	    </dia:attribute>
	    <dia:attribute name="scaling">
	      <dia:real val="0.25"/>
	    </dia:attribute>
	    <dia:attribute name="fitto">
	      <dia:boolean val="false"/>
	    </dia:attribute>
	  </dia:composite>
	</dia:attribute>
	<dia:attribute name="grid">
	  <dia:composite type="grid">
	    <dia:attribute name="width_x">
	      <dia:real val="1"/>
	    </dia:attribute>
	    <dia:attribute name="width_y">
	      <dia:real val="1"/>
	    </dia:attribute>
	    <dia:attribute name="visible_x">
	      <dia:int val="1"/>
	    </dia:attribute>
	    <dia:attribute name="visible_y">
	      <dia:int val="1"/>
	    </dia:attribute>
	  </dia:composite>
	</dia:attribute>
	<dia:attribute name="guides">
	  <dia:composite type="guides">
	    <dia:attribute name="hguides"/>
	    <dia:attribute name="vguides"/>
	  </dia:composite>
	</dia:attribute>
      </dia:diagramdata>
      <dia:layer name="Background" visible="true">

	<xsl:apply-templates/>

      </dia:layer>
    </dia:diagram>
  </xsl:template>

<!-- ========================================================= -->

  <xsl:template match="uxf:Class">
    <dia:object type="UML - Class" version="0" id="{uxf:Name}">

      <dia:attribute name="elem_corner">
        <xsl:variable name="pos-tv"
		      select="uxf:TaggedValue[string(uxf:Tag)='DiagramPosition']"/>
        <xsl:choose>
          <xsl:when test="$pos-tv">
            <dia:point val="{$pos-tv/uxf:Value}"/>
          </xsl:when>
          <xsl:otherwise>
            <dia:point val="{7.0+position()},{7.0+position()}"/>
          </xsl:otherwise>
        </xsl:choose>
      </dia:attribute>

      <dia:attribute name="name">
        <dia:string>#<xsl:value-of select="uxf:Name"/>#</dia:string>
      </dia:attribute>

      <dia:attribute name="stereotype">
        <dia:string>
	  <xsl:if test="uxf:Stereotype/uxf:Name">
            <xsl:text>#</xsl:text>
            <xsl:value-of select="uxf:Stereotype/uxf:Name"/>
            <xsl:text>#</xsl:text>
          </xsl:if>
	</dia:string>
      </dia:attribute>

      <dia:attribute name="abstract">
        <dia:boolean val="{uxf:IsAbstract}"/>
      </dia:attribute>

      <!-- IsActive has no representation in dia -->
      <!-- TODO: put a warning to the resulting diagram if it is
      present in the UXF file -->


      <dia:attribute name="suppress_attributes">
        <dia:boolean val="false"/>
      </dia:attribute>
      <dia:attribute name="suppress_operations">
        <dia:boolean val="false"/>
      </dia:attribute>
      <dia:attribute name="visible_attributes">
        <dia:boolean val="{$members = 1}"/>
      </dia:attribute>
      <dia:attribute name="visible_operations">
        <dia:boolean val="{$members = 1}"/>
      </dia:attribute>

      <dia:attribute name="attributes">
       <xsl:if test="$members = 1">

        <xsl:for-each select="uxf:Attribute">
          <dia:composite type="umlattribute">

            <dia:attribute name="name">
              <dia:string>
                <xsl:text>#</xsl:text>
                <xsl:value-of select="uxf:Name"/>
                <xsl:if test="uxf:Multiplicity and $attr-multiplicities = 1">
                  <xsl:text>[</xsl:text> 
                  <xsl:value-of select="uxf:Multiplicity"/>
                  <xsl:text>]</xsl:text> 
                  </xsl:if>
                <xsl:text>#</xsl:text>
              </dia:string>
            </dia:attribute>

            <dia:attribute name="type">
              <dia:string>#<xsl:value-of select="uxf:Type"/>#</dia:string>
            </dia:attribute>

            <dia:attribute name="value">
              <dia:string>
                <xsl:if test="uxf:InitialValue">#<xsl:value-of
                      select="uxf:InitialValue"/>#</xsl:if>
              </dia:string>
            </dia:attribute>

            <xsl:apply-templates select="uxf:Visibility"/>

            <dia:attribute name="class_scope">
              <!-- UXF bug -->
              <dia:boolean val="{string(uxf:TargetScope)='classifier'}"/>
            </dia:attribute>
          </dia:composite>
        </xsl:for-each>

       </xsl:if>
      </dia:attribute>

      <dia:attribute name="operations">
       <xsl:if test="$members = 1">

	<xsl:for-each select="uxf:Operation">
	  <dia:composite type="umloperation">

	    <dia:attribute name="name">
	      <dia:string>#<xsl:value-of select="uxf:Name"/>#</dia:string>
	    </dia:attribute>

	    <dia:attribute name="type">
	      <dia:string>
                <xsl:variable name="query-tv"
			      select="uxf:TaggedValue[string(uxf:Tag)='IsQuery']"/>
		<xsl:if test="uxf:ReturnType or 
			($query-tv and
			string($query-tv/uxf:Value)='true')">
                  <xsl:text>#</xsl:text>
                  <xsl:value-of select="uxf:ReturnType"/>
                  <xsl:if test="$query-tv and string($query-tv/uxf:Value)='true'">
                    <xsl:text> {query}</xsl:text>
                  </xsl:if>
                  <xsl:text>#</xsl:text>
                </xsl:if>
              </dia:string>
	    </dia:attribute>

	    <dia:attribute name="abstract">
	      <dia:boolean val="{uxf:IsAbstract}"/>
	    </dia:attribute>

	    <dia:attribute name="class_scope">
	      <dia:boolean val="{string(uxf:OwnerScope)='classifier'}"/>
	    </dia:attribute>

	    <xsl:apply-templates select="uxf:Visibility"/>

	    <dia:attribute name="parameters">
              <xsl:choose>

		<xsl:when test="string-length(.) &gt; $max-param-len">
                  <dia:composite type="umlparameter">
                     <xsl:message>
                       <xsl:text>Parameter list too long (</xsl:text>
                       <xsl:value-of select="../uxf:Name"/>
                       <xsl:text>::</xsl:text>
                       <xsl:value-of select="uxf:Name"/>
                       <xsl:text>: </xsl:text>
                       <xsl:value-of select="string-length(.)"/>
                       <xsl:text>), not shown.</xsl:text>
                     </xsl:message>
                     <dia:attribute name="name">
                       <dia:string>#$$$#</dia:string>
                     </dia:attribute>
                     <dia:attribute name="type">
                       <dia:string>##</dia:string>
                     </dia:attribute>
                     <dia:attribute name="value">
                       <dia:string/>
                     </dia:attribute>
                     <!-- duh, Dia chokes on comments :( -->
                     <xsl:comment>Box would be too wide</xsl:comment>
                  </dia:composite>
		</xsl:when>

		<xsl:otherwise>
		  <xsl:for-each select="uxf:Parameter">
		    <dia:composite type="umlparameter">
		      <xsl:call-template name="name-type"/>
		      <dia:attribute name="value">
			<dia:string>
			  <xsl:if test="uxf:DefaultValue">#<xsl:value-of 
                                select="uxf:DefaultValue"/>#</xsl:if>
			</dia:string>
		      </dia:attribute>
		      <!-- TODO: Kind - dia has some code for it but
		      it doesn't use it -->
		    </dia:composite>
		  </xsl:for-each>
		</xsl:otherwise>
              </xsl:choose>

	    </dia:attribute>
	  </dia:composite>
        </xsl:for-each>
       </xsl:if> 
      </dia:attribute>

      <dia:attribute name="template">
        <dia:boolean>
          <xsl:attribute name="val">
            <xsl:choose>
               <xsl:when test="uxf:TemplateParameters">true</xsl:when>
               <xsl:otherwise>false</xsl:otherwise>
            </xsl:choose>
          </xsl:attribute>
        </dia:boolean>
      </dia:attribute>

      <dia:attribute name="templates">
        <xsl:for-each select="uxf:TemplateParameters/uxf:Parameter">
          <dia:composite type="umlformalparameter">
            <xsl:call-template name="name-type"/>

            <!-- DefaultValue has no representation in dia -->
            <!-- Kind does not make sense - probably an
            oversimplifiation in UXF DTD -->

          </dia:composite>
        </xsl:for-each>
      </dia:attribute>
    </dia:object>
  </xsl:template>


  <xsl:template name="name-type">
      <dia:attribute name="name">
        <dia:string>#<xsl:value-of select="uxf:Name"/>#</dia:string>
      </dia:attribute>
      <dia:attribute name="type">
        <dia:string>#<xsl:value-of select="uxf:Type"/>#</dia:string>
      </dia:attribute>
  </xsl:template>

  <xsl:template match="uxf:Visibility">
      <dia:attribute name="visibility">
         <dia:enum>
            <xsl:attribute name="val">
            <xsl:choose>
               <xsl:when test="string(.)='public'">0</xsl:when>
               <xsl:when test="string(.)='private'">1</xsl:when>
               <xsl:when test="string(.)='protected'">2</xsl:when>
            </xsl:choose>
            </xsl:attribute>
         </dia:enum>
      </dia:attribute>
  </xsl:template>

<!-- ========================================================= -->

  <xsl:template match="uxf:Association">
    <dia:object type="UML - Association" version="0" id="{generate-id(.)}">
      <xsl:call-template name="hvh-route"/>

      <dia:attribute name="name">
        <dia:string>#<xsl:value-of select="uxf:AssociationName/uxf:Name"/>#</dia:string>
      </dia:attribute>

      <dia:attribute name="direction">
        <dia:enum val="0"/> <!-- right-left is stupid -->
      </dia:attribute>

      <dia:attribute name="ends">
       <xsl:for-each select="uxf:AssociationEnd">
        <dia:composite>
          <xsl:variable name="visi-prefix">
            <xsl:choose>
               <xsl:when test="string(Visibility)='public'">+</xsl:when>
               <xsl:when test="string(Visibility)='private'">-</xsl:when>
               <xsl:when test="string(Visibility)='protected'">#</xsl:when>
            </xsl:choose>
          </xsl:variable>

          <dia:attribute name="role">
            <dia:string>#<xsl:value-of select="concat($visi-prefix,uxf:Name)"
				   />#</dia:string>
          </dia:attribute>

          <dia:attribute name="multiplicity">
            <dia:string>
              <xsl:text>#</xsl:text>
              <xsl:value-of select="uxf:Multiplicity"/>

              <xsl:if test="uxf:Ordering">
                <xsl:text> {</xsl:text>
                  <xsl:value-of select="uxf:Ordering"/>
                <xsl:text>}</xsl:text>
              </xsl:if>
              <xsl:text>#</xsl:text>
            </dia:string>
          </dia:attribute>

          <dia:attribute name="arrow">
            <dia:boolean val="false"/>
            <!-- anyway, if it is navigable, there is always a role name -->
<!--
            <dia:boolean val="{uxf:isNavigable}"/>
-->
          </dia:attribute>

          <dia:attribute name="aggregate">
           <dia:enum>
            <xsl:attribute name="val">
             <xsl:choose>
               <xsl:when test="string(uxf:AggregationKind)
			       ='AggregationWhole'">1</xsl:when>
               <xsl:when test="string(uxf:AggregationKind)
                               ='CompositionWhole'">2</xsl:when>
               <xsl:otherwise>0</xsl:otherwise>
             </xsl:choose>
            </xsl:attribute>
           </dia:enum>
          </dia:attribute>
        </dia:composite>
       </xsl:for-each>
      </dia:attribute>

      <dia:connections>
        <xsl:variable name="conn1-tv"
		      select="uxf:TaggedValue[string(uxf:Tag)='DiagramConnection1']"/>
        <xsl:variable name="conn2-tv"
		      select="uxf:TaggedValue[string(uxf:Tag)='DiagramConnection2']"/>

        <xsl:choose>
          <xsl:when test="$conn1-tv">
            <dia:connection handle="0" to="{uxf:AssociationEnd[1]/uxf:Type}" 
			connection="{$conn1-tv/uxf:Value}"/>
          </xsl:when>
          <xsl:otherwise>
            <dia:connection handle="0" to="{uxf:AssociationEnd[1]/uxf:Type}" 
			connection="0"/> <!-- 0 is top left -->
          </xsl:otherwise>
        </xsl:choose>
        <xsl:choose>
          <xsl:when test="$conn2-tv">
            <dia:connection handle="1" to="{uxf:AssociationEnd[2]/uxf:Type}" 
			connection="{$conn2-tv/uxf:Value}"/>
          </xsl:when>
          <xsl:otherwise>
            <dia:connection handle="1" to="{uxf:AssociationEnd[2]/uxf:Type}" 
			connection="2"/> <!-- 2 is top right -->
          </xsl:otherwise>
        </xsl:choose>
      </dia:connections>
    </dia:object>
  </xsl:template>

<!-- ========================================================= -->
  <!-- key('classkey',...) ensures that the linked objects exist -->
  <xsl:template match="uxf:Generalization[key('classkey',string(uxf:Parent)) and
		                      key('classkey',string(uxf:Child))]">
    <dia:object type="UML - Generalization" version="0" id="{generate-id(.)}">
      <xsl:call-template name="vhv-route"/>

      <dia:attribute name="name">
        <!-- UXF doesn't provide a name -->
        <dia:string/>
      </dia:attribute>

      <dia:attribute name="stereotype">
        <dia:string>
	  <xsl:if test="uxf:Stereotype/uxf:Name">
            <xsl:text>#</xsl:text>
            <xsl:value-of select="uxf:Stereotype/uxf:Name"/>
            <xsl:text>#</xsl:text>
          </xsl:if>
	</dia:string>
      </dia:attribute>

      <dia:connections>
        <!-- connection 6 is bottom middle, 1 is top middle -->
        <dia:connection handle="0" to="{uxf:Parent}" connection="6"/>
        <dia:connection handle="1" to="{uxf:Child}" connection="1"/>
      </dia:connections>
    </dia:object>
  </xsl:template>

<!-- ========================================================= -->

  <!-- a vertical-horizontal-vertical route -->
  <xsl:template name="vhv-route">
    <xsl:variable name="route-tv"
		  select="uxf:TaggedValue[string(uxf:Tag)='DiagramRoute']"/>
    <xsl:choose>
      <xsl:when test="$route-tv">
        <xsl:copy-of select="$route-tv/uxf:Value/*"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:call-template name="default-vhv-route"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <!-- a horizontal-vertical-horizontal route -->
  <xsl:template name="hvh-route">
    <xsl:variable name="route-tv"
		  select="uxf:TaggedValue[string(uxf:Tag)='DiagramRoute']"/>
    <xsl:choose>
      <xsl:when test="$route-tv">
        <xsl:copy-of select="$route-tv/uxf:Value/*"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:call-template name="default-hvh-route"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template name="default-vhv-route">
    <dia:attribute name="orth_points">
      <dia:point val="7.0,7.0"/>
      <dia:point val="7.0,-2"/>
      <dia:point val="3.7,-2"/>
      <dia:point val="3.7,8.1"/>
    </dia:attribute>
    <dia:attribute name="orth_orient">
      <dia:enum val="1"/>
      <dia:enum val="0"/>
      <dia:enum val="1"/>
    </dia:attribute>
  </xsl:template>

  <xsl:template name="default-hvh-route">
    <dia:attribute name="orth_points">
      <dia:point val="7.0,7.0"/>
      <dia:point val="-3,7.0"/>
      <dia:point val="-3,8"/>
      <dia:point val="3,8"/>
    </dia:attribute>
    <dia:attribute name="orth_orient">
      <dia:enum val="0"/>
      <dia:enum val="1"/>
      <dia:enum val="0"/>
    </dia:attribute>
  </xsl:template>

<!-- ========================================================= -->

<!--
-->
</xsl:stylesheet>
